import { ParallelStructure } from "./parallel_structure";

// Classes générales sur les structures
import { RectangularStructureParams } from "./rectangular_structure_params";
import { Structure } from "./structure";
import { LoiDebit } from "./structure_props";

// Equations de débit
import { StructureGateCem88d, StructureWeirCem88d } from "./structure_cem88d";
import { StructureGateCem88v, StructureWeirCem88v } from "./structure_cem88v";
import { StructureCunge80 } from "./structure_cunge80";
import { StructureKivi, StructureKiviParams } from "./structure_kivi";
import { StructureOrificeSubmerged, StructureOrificeSubmergedParams } from "./structure_orifice_submerged";
import { StructureRectangularOrificeFree } from "./structure_rectangular_orifice_free";
import { StructureRectangularOrificeSubmerged } from "./structure_rectangular_orifice_submerged";
// tslint:disable-next-line:max-line-length
import { StructureTriangularTruncWeirFree, TriangularTruncStructureParams } from "./structure_triangular_trunc_weir_free";
import { StructureTriangularWeirFree, TriangularStructureParams } from "./structure_triangular_weir_free";
import { StructureWeirFree } from "./structure_weir_free";
import { StructureWeirSubmergedLarinier } from "./structure_weir_submerged_larinier";

export function CreateStructure(loiDebit: LoiDebit, parentNub?: ParallelStructure,
                                dbg: boolean = false): Structure {
    const oCd: {[s: string]: number} = {SeuilR: 0.4, VanneR: 0.6, SeuilT: 1.36};
    const rectStructPrms: RectangularStructureParams = new RectangularStructureParams(
        0,  // Q
        100,        // ZDV
        102,        // Z1
        101.5,      // Z2
        2,          // L
        oCd.SeuilR        // Cd pour un seuil rectangulaire
        // W = Infinity par défaut pour un seuil
    );

    // Control of validity of structure type and definition of parameters
    switch (loiDebit) {
        case LoiDebit.GateCem88d:
        case LoiDebit.GateCem88v:
        case LoiDebit.Cunge80:
        case LoiDebit.RectangularOrificeFree:
        case LoiDebit.RectangularOrificeSubmerged:
            rectStructPrms.W.singleValue = 0.5;
            rectStructPrms.Cd.v = oCd.VanneR; // Cd pour une vanne rectangulaire
    }

    // Instanciation of the equation
    switch (loiDebit) {
        case LoiDebit.WeirCem88d:
            rectStructPrms.Cd.v = oCd.SeuilR; // Cd Cem88d en seuil
            return new StructureWeirCem88d(rectStructPrms, dbg);

        case LoiDebit.WeirCem88v:
            rectStructPrms.Cd.v = oCd.VanneR; // Cd Cem88d en seuil
            return new StructureWeirCem88v(rectStructPrms, dbg);

        case LoiDebit.GateCem88d:
            rectStructPrms.Cd.v = oCd.SeuilR; // Cd Cem88d en vanne
            return new StructureGateCem88d(rectStructPrms, dbg);

        case LoiDebit.GateCem88v:
            rectStructPrms.Cd.v = oCd.VanneR; // Cd Cem88d en vanne
            return new StructureGateCem88v(rectStructPrms, dbg);

        case LoiDebit.Cunge80:
            rectStructPrms.Cd.v = oCd.VanneR; // Cd Cunge80 qu'on soit en seuil ou vanne
            return new StructureCunge80(rectStructPrms, dbg);

        case LoiDebit.RectangularOrificeFree:
            return new StructureRectangularOrificeFree(rectStructPrms, dbg);

        case LoiDebit.RectangularOrificeSubmerged:
            return new StructureRectangularOrificeSubmerged(rectStructPrms, dbg);

        case LoiDebit.WeirFree:
            return new StructureWeirFree(rectStructPrms, dbg);

        case LoiDebit.WeirSubmergedLarinier:
            return new StructureWeirSubmergedLarinier(rectStructPrms, dbg);

        case LoiDebit.KIVI:
            const structKiviPrm: StructureKiviParams = new StructureKiviParams(
                8.516,      // Q
                101.5,        // ZDV
                103,        // Z1
                102,        // Z2
                2,          // L
                oCd.SeuilR, // alpha
                0.001,      // béta
                100);       // ZRAM : cote Radier Amont
            return new StructureKivi(structKiviPrm, dbg);
        case LoiDebit.TriangularWeirFree:
            const structTriangPrms: TriangularStructureParams = new TriangularStructureParams(
                0,          // Q
                100,        // ZDV
                102,        // Z1
                45,         // Alpha2
                oCd.SeuilT  // Cd pour un seuil triangulaire
                // W = Infinity par défaut pour un seuil
            );
            return new StructureTriangularWeirFree(structTriangPrms, dbg);
        case LoiDebit.TriangularTruncWeirFree:
            const structTriTruncPrms: TriangularTruncStructureParams = new TriangularTruncStructureParams(
                0,          // Q
                100.1,        // ZDV
                102,        // Z1
                0.9,        // BT
                101,        // ZT
                oCd.SeuilT  // Cd pour un seuil triangulaire
                // W = Infinity par défaut pour un seuil
            );
            return new StructureTriangularTruncWeirFree(structTriTruncPrms, dbg);
        case LoiDebit.OrificeSubmerged:
            return new StructureOrificeSubmerged(
                new StructureOrificeSubmergedParams(
                    0,      // Q
                    102,    // Z1
                    101.5,  // Z2
                    0.7,    // Cd
                    0.1     // S
                ),
                dbg
            );

        default:
            throw new Error(`type de LoiDebit ${LoiDebit[loiDebit]} non pris en charge`);
    }
}
